"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EventExecutor_1 = require("./event/EventExecutor");
class Aggregate {
    static applyEvents(allEvents, eventExecutors, aggregateClasses) {
        const firstEvent = allEvents[0];
        const firstEventExecutor = EventExecutor_1.EventExecutor.getEventExecutor(firstEvent, eventExecutors);
        if (!firstEventExecutor) {
            throw new Error(`Unknown event during getAggregate: ${firstEvent.constructor.name}`);
        }
        if (!firstEventExecutor || !firstEventExecutor.isAggregateCreating()) {
            throw new Error(`The first event of aggregate ${firstEventExecutor.getAggregateId(firstEvent)} is not an AggregateCreatingEvent. Panic!`);
        }
        const aggregate = Aggregate.instantiateAggregate(firstEventExecutor.getAggregateType(), firstEventExecutor.getAggregateId(firstEvent), aggregateClasses);
        allEvents.forEach((e) => {
            const eventExecutor = EventExecutor_1.EventExecutor.getEventExecutor(e, eventExecutors);
            Aggregate.applyEventOnAggregate(e, eventExecutor, aggregate);
        });
        return aggregate;
    }
    static applyEventOnAggregate(event, eventExecutor, aggregate) {
        if (eventExecutor) {
            aggregate.checkEventCanBeApplied(event, eventExecutor.getEventOrdinal(event), eventExecutor.getEventId(event));
            // applying event data to aggregate data.
            eventExecutor.apply(event, aggregate);
            const eventOrdinal = eventExecutor.getEventOrdinal(event);
            if (eventOrdinal) {
                aggregate.setMaxEventOrdinal(eventOrdinal);
            }
            else {
                aggregate.setMaxEventOrdinal(aggregate.getNextEventOrdinal());
            }
        }
        else {
            throw new Error(`Unknown event during getEventExecutor: ${event.constructor.name}`);
        }
    }
    static instantiateAggregate(aggregateType, aggregateId, aggregateClasses) {
        const aggregateClass = aggregateClasses.has(aggregateType) ? aggregateClasses.get(aggregateType) : Aggregate;
        return new aggregateClass(aggregateType, aggregateId);
    }
    constructor(aggregateType, aggregateId) {
        this.aggregateType = aggregateType;
        this.aggregateId = aggregateId;
        this.aggregateRoles = new Map();
        this.maxEventOrdinal = 0;
    }
    getAggregateType() {
        return this.aggregateType;
    }
    getAggregateId() {
        return this.aggregateId;
    }
    getFullId() {
        return `${this.aggregateType}:${this.aggregateId}`;
    }
    /**
     * Gets the extra roles that this aggregate grants
     */
    getAggregateRoles() {
        return this.aggregateRoles;
    }
    /**
     * Gets the extra roles that this aggregate grants to an entity
     * @param {string} Optional. The id of the entity we're asking about.
     * @returns {[string]} The list of roles for this aggregate
     */
    getAggregateRolesFor(entityId) {
        if (this.aggregateRoles.has(entityId)) {
            return this.aggregateRoles.get(entityId);
        }
        return [];
    }
    addAggregateRole(entityId, roles) {
        if (this.aggregateRoles.has(entityId)) {
            this.aggregateRoles.set(entityId, this.aggregateRoles.get(entityId).concat(roles));
        }
        else {
            this.aggregateRoles.set(entityId, roles);
        }
    }
    removeAggregateRole(entityId, roles) {
        if (!this.aggregateRoles.has(entityId)) {
            return; // Nothing to do.
        }
        if (!roles) {
            this.aggregateRoles.delete(entityId);
        }
        else {
            const entityRoles = this.aggregateRoles.get(entityId);
            const newRoles = entityRoles.filter((role) => roles.indexOf(role) < 0);
            this.aggregateRoles.set(entityId, newRoles);
        }
    }
    /**
     * Check that this event can be applied to this aggregate as the ordinal is the next one
     *
     * @param event
     * @returns number
     */
    checkEventCanBeApplied(event, eventOrdinal, eventId) {
        if (!eventOrdinal) {
            return; // Nothing to check
        }
        // if event has not applied ordinal.
        if (this.isPastOrdinal(eventOrdinal)) {
            const aggregateId = this.getAggregateId();
            const msg = `Event (${eventId}) has already been applied to aggregate (${aggregateId}). Its current maxOrdinal is ${this.maxEventOrdinal}`;
            throw new Error(msg);
        }
        if (eventOrdinal !== this.getNextEventOrdinal()) {
            const aggregateId = this.getAggregateId();
            throw new Error(`Applying non consecutive event (eventId: ${eventId}, ordinal: ${eventOrdinal}) to the aggregate (aggregateId: ${aggregateId}, maxOrdinal: ${this.maxEventOrdinal})`);
        }
    }
    /**
     * Use event ordinal to check if an event has been applied to the aggregate.
     * @param eventOrdinal
     * @returns boolean
     */
    isPastOrdinal(eventOrdinal) {
        return eventOrdinal > 0 && !(this.maxEventOrdinal < eventOrdinal);
    }
    /**
     * Use eventCount if its great than 0 or use eventsTotal
     *
     * @returns number
     */
    getNextEventOrdinal() {
        return this.getMaxEventOrdinal() + 1;
    }
    getMaxEventOrdinal() {
        return this.maxEventOrdinal;
    }
    setMaxEventOrdinal(maxEventOrdinal) {
        this.maxEventOrdinal = maxEventOrdinal;
    }
}
exports.Aggregate = Aggregate;
//# sourceMappingURL=Aggregate.js.map